/*------------------------------------------------------------------
metronome Version 2 for NJU3711D

 Device   : ATtiny13A
 Clock    : 9.6MHz
 FUSE     : -FL01111010

 Motohiko Takeda

 pin1 reset(open)
 pin2 Level 1
 pin3 
 pin4 GND
 pin5 LD
 pin6 CLK
 pin7 DATA
 pin7 VCC

------------------------------------------------------------------ */
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include "tempo_curve.h"


#define LATCH		PB0
#define CLK			PB1
#define DATA		PB2

#define CLK_HI	(PORTB |= (1 << CLK))
#define CLK_LOW	(PORTB &= ~(1 << CLK))

#define DATA_HI	 (PORTB |= (1 << DATA))
#define DATA_LOW (PORTB &= ~(1 << DATA))

#define ADC_3CH ((1<< MUX1) | (1<<MUX0))	//adc3

volatile uint8_t ADC_value = 255 ;

volatile  uint8_t LFO_SPEED = 252;
volatile  int8_t flag = 1 ;
volatile  int8_t pulse;
 
ISR (ADC_vect)
{
	//ADC clk = 75kHz 13clk1ADC->5.7k(170uSɃGg[)
	ADC_value = ADCH ;
	ADMUX |= ADC_3CH  ;					//ch ݒ

	ADCSRA |= (1 << ADSC ) ;			//ADC start
}

void pulse_on(void)
{
	pulse = 187;
	PORTB |= (1 << PB4);
}

ISR (TIM0_OVF_vect)
{
//	9.6MHz 256*3(80uS)ƂɌĂяoO	31->30BPM(12Hz 24Tick) 251->240bpm(96Hz 24Ticks)

	#define MAGIC_NUM 32768 ;
	static int16_t accumulator = MAGIC_NUM;	//accumulator for LFO
	static uint8_t pre_scale = 0; 

	if ( ++ pre_scale == 3 ){
		accumulator -= (int16_t)LFO_SPEED ;
		if (accumulator < 0){
			accumulator += MAGIC_NUM;
			if ( flag == 0 ){
				flag++;
			}
		}
		pre_scale = 0;

		if ( pulse != 0 ){
			if ( -- pulse == 0 ){
				PORTB &= ~(1 << PB4);
			}
		}
	}
}


int8_t SpeedConvert4LFO_exp16(uint8_t in)
{
	uint16_t work;									//16BitȓɎ߂ׂTCrbgvZɎg
	int16_t v;
	int16_t integer ;
	int16_t fractional ;


	//form 0 to 256 ->  from 31 to 252
	// int (Log(31,2) * 64 )= 318
	// int (Log(252,2) * 64 )= 510
													
	work =  ( ( (uint16_t)in * (uint16_t)((510-318)/2) ) >> 7 ) + 318;

	v = ( work >> 6 ) -1  ;
	integer  = 2 << ( v ) ;							// vZ	 pow(2, temp/64 ) 
													// 搔0̎̏ȂBv  
	fractional = (work & 0x003F) + 64 ; 			// _ȉ͒ߎ
	work = integer * fractional ;					// Ə_ȉ
	work >>= 6;										// _ȉ؎̂ĂĐ

	if (work < 31 ){
		return 31;
	}
	if (work > 252 ) {
		return 252;
	}
	return work;
}


void send_data(uint8_t val )
{
	uint8_t i;
	uint8_t mask = 0x80;

	asm volatile("nop");	//		wait_1us();
	for (i = 0 ; i < 8; i++){
		if ( mask & val){
			DATA_LOW;
		}else{
			DATA_HI;
		}
//		asm volatile("nop");	//		wait_1us();
		CLK_HI;
//		asm volatile("nop");	//		wait_1us();
		CLK_LOW;
		mask >>= 1;
	}
}

void init()
{
	// PB0  PB1 for SW 
	DDRB |=  (1 << LATCH) | (1 << CLK) | (1 << DATA);

	DDRB |=  (1 << PB4);

	ADCSRA |= (1 << ADEN)|(1 << ADIE) | (1 << ADPS2) |(1 << ADPS1)  |(1 << ADPS0) ;	// 9.6MHz * 1/128 (75kHz)
	ADMUX  |= (1 << ADLAR);						// upper 8bit only
	DIDR0  |= (1 << ADC3D ) ;		// ADC2:speed1, ADC3:speed2

	TCCR0A|= (1 << COM0A1) | (1 << COM0A0) | (1 << WGM01) | (1 << WGM00);
   
	TCCR0B|= (1 << CS00) ;
	TIMSK0|= (1 << TOIE0); 
	OCR0A = 255;

	sei();								// 荞݋

	ADMUX &= ~(0x03) ;					//MUX clear
	ADMUX |= ADC_3CH  ;					//ch ݒ
	ADCSRA |= (1 << ADSC ) ;			//ADC start

	CLK_LOW;
	DATA_LOW;

}

uint8_t pattern[] = {1, 2, 4, 8 };


uint8_t brightness[] = { 
0,
0,
0,
19,
38,
56,
75,
93,
110,
127,
143,
158,
173,
186,
199,
210,
220,
229,
237,
243,
248,
252,
254,
255
};


int main(void)
{
	#define FLASH_LENGTH 2

	int8_t flash_beat =1;

	uint8_t work;
	uint8_t base_cnt = 0;
	uint8_t bar_cnt = 0;
	uint8_t note_cnt = 0;

	init();

	for (; ;){
		if (flag) {
			flag = 0;
			if ( base_cnt < FLASH_LENGTH ){
				if ( base_cnt == 0 ){
					send_data(255);
				}
			}else{
				if ( flash_beat ){
					flash_beat = 0;
					work  = pattern[ note_cnt ];
					work |= pattern[ 3 - bar_cnt ] << 4;
//					work |= pattern[ bar_cnt ] << 4;
					send_data(work);
				}
			}
			OCR0A = brightness[base_cnt];

			if ( base_cnt == 0 || base_cnt == 6 || base_cnt == 12 || base_cnt == 18 ){
				pulse_on();
			}

			if ( ++ base_cnt == 24){
				base_cnt = 0;
				flash_beat++;
				if ( ++ note_cnt == 4 ){
					note_cnt = 0;
					if ( ++ bar_cnt == 4 ){
						bar_cnt = 0;
					}
				}
			}
			LFO_SPEED = pgm_read_byte(&tempo_curve[ (uint8_t)(ADC_value)  ]);
		}
	}
}
